/*
 * This file was generated automatically by xsubpp version 1.9505 from the 
 * contents of smbauth.xs. Do not edit this file, edit smbauth.xs instead.
 *
 *	ANY CHANGES MADE HERE WILL BE LOST! 
 *
 */

#line 1 "smbauth.xs"
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

XS(XS_Authen__Smb_Valid_User)
{
    dXSARGS;
    if (items != 5)
	croak("Usage: Authen::Smb::Valid_User(username, password, server, backup, domain)");
    {
	char*	username = (char *)SvPV(ST(0),na);
	char*	password = (char *)SvPV(ST(1),na);
	char*	server = (char *)SvPV(ST(2),na);
	char*	backup = (char *)SvPV(ST(3),na);
	char*	domain = (char *)SvPV(ST(4),na);
	int	RETVAL;

	RETVAL = Valid_User(username, password, server, backup, domain);
	ST(0) = sv_newmortal();
	sv_setiv(ST(0), (IV)RETVAL);
    }
    XSRETURN(1);
}

#ifdef __cplusplus
extern "C"
#endif
XS(boot_Authen__Smb)
{
    dXSARGS;
    char* file = __FILE__;

    XS_VERSION_BOOTCHECK ;

        newXS("Authen::Smb::Valid_User", XS_Authen__Smb_Valid_User, file);
    ST(0) = &sv_yes;
    XSRETURN(1);
}
